@@ -43,7 +43,9 @@ pub enum FusedSpec<'t> {
     BinPerCol(&'t Tensor, BinOp),
     AddRowColProducts(&'t Tensor, &'t Tensor),
     AddUnicast(OutputStore),
-    QScale(usize, RoundingPolicy, i32),
+    QScale(isize, RoundingPolicy, i32),
+    RoundingShiftRight(usize, RoundingPolicy),
+    ShiftLeft(usize),
     Store(OutputStore),
     AddMatMul { k: usize, a: PackedStore, b: InputStore },
 }
@@ -91,7 +93,9 @@ pub enum FusedKerSpec<TI: Copy> {
     PerColSub(*const TI),                       // jump_to:per_col_sub
     PerColSubF(*const TI),                      // jump_to:per_col_sub_flipped
 
-    QScale(usize, RoundingPolicy, i32),         // jump_to:q_scale
+    QScale(isize, RoundingPolicy, i32),         // jump_to:q_scale
+    RoundingShiftRight(usize, RoundingPolicy),  // jump_to:q_shr
+    ShiftLeft(usize),                           // jump_to:q_shl
     AddUnicast(OutputStoreKer),                 // jump_to:add_unicast
     AddRowColProducts(*const TI, *const TI),    // jump_to:add_row_col_products
     Store(OutputStoreKer),                      // jump_to:store
@@ -105,7 +109,7 @@ pub enum FusedKerSpec<TI: Copy> {
 pub mod test {
     use crate::frame::mmm::storage::*;
     use crate::frame::mmm::*;
-    use crate::generic::ScaleShiftAndRound;
+    use crate::generic::{ScaleShiftAndRound, Scaler};
     use num_traits::{AsPrimitive, Bounded};
     use proptest::prelude::*;
     use tract_data::internal::*;
@@ -127,10 +131,10 @@ pub mod test {
     macro_rules! mmm_kernel_fuse_tests {
         ($cond:expr, $ker:ident, $tc:ty, $ti: ty) => {
             mod fuse {
+                use super::super::$ker;
                 #[allow(unused_imports)]
                 use crate::frame::mmm::fuse::test;
                 use crate::frame::mmm::fuse::test::tile;
-                use super::super::$ker;
 
                 #[test]
                 fn return_zeros() {
@@ -272,77 +276,66 @@ pub mod test {
                 use crate::frame::mmm::fuse::test;
                 use crate::frame::mmm::fuse::test::QScaleProblem;
                 use crate::frame::mmm::kernel::MatMatMulKer;
+                use crate::generic::Scaler;
                 use proptest::prelude::*;
                 use super::super::$ker;
 
-                #[test]
-                fn return_q_scale_0() {
-                    if $cond {
-                        let len = <$ker>::mr() * <$ker>::nr();
-                        let mut v = vec!(0; len - 1);
-                        v.push(1);
-                        QScaleProblem::<$ker, $tc, $ti>::new(v, 2^31, 1, RoundingPolicy::Zero).run()
-                    }
-                }
-
-                #[test]
-                fn return_q_scale_1() {
-                    if $cond {
-                        let len = <$ker>::mr() * <$ker>::nr();
-                        let mut v = vec!(0; len - 1);
-                        v.push(1);
-                        QScaleProblem::<$ker, $tc, $ti>::new(v, 2^31, 1, RoundingPolicy::Away).run()
-                    }
-                }
-
-                #[test]
-                fn return_q_scale_2() {
-                    if $cond {
-                        let len = <$ker>::mr() * <$ker>::nr();
-                        let mut v = vec!(0; len - 1);
-                        v.push(4);
-                        QScaleProblem::<$ker, $tc, $ti>::new(v, 2^31, 3, RoundingPolicy::Odd).run()
-                    }
-                }
-
-                #[test]
-                fn return_q_scale_3() {
-                    if $cond {
-                        let len = <$ker>::mr() * <$ker>::nr();
-                        let mut v = vec!(0; len - 1);
-                        v.push(1);
-                        QScaleProblem::<$ker, $tc, $ti>::new(v, 536870913, 0, RoundingPolicy::Zero).run()
-                    }
-                }
-                #[test]
-                fn return_q_scale_4() {
-                    if $cond {
-                        let len = <$ker>::mr() * <$ker>::nr();
-                        let mut v = vec!(0; len - 1);
-                        v.push(2);
-                        QScaleProblem::<$ker, $tc, $ti>::new(v, 536870913, 0, RoundingPolicy::Zero).run()
-                    }
-                }
-
-                #[test]
-                fn return_q_scale_5() {
-                    if $cond {
-                        let len = <$ker>::mr() * <$ker>::nr();
-                        let mut v = vec!(0; len - 1);
-                        v.push(6);
-                        QScaleProblem::<$ker, $tc, $ti>::new(v, 715827883, 0, RoundingPolicy::Even).run()
-                    }
-                }
-
+                // FIXME: Scaler should be arbitrary
                 macro_rules! test_q_scale {
                     ($policy: ident) => {
                         paste! {
                             #[test]
-                            fn [<return_q_scale_ $policy:lower>]() {
+                            fn [<return_q_scale_halfpos_ $policy:lower>]() {
                                 if $cond {
                                     let len = (<$ker>::mr() * <$ker>::nr()) as i64;
                                     let v = (0..len).map(|i| (i - len / 2) as $tc).collect();
-                                    QScaleProblem::<$ker, $tc, $ti>::new(v, 1<<30, 2, RoundingPolicy::$policy).run()
+                                    QScaleProblem::<$ker, $tc, $ti>::new(v, Scaler::new(0.5f32, RoundingPolicy::$policy)).run()
+                                }
+                            }
+
+                            #[test]
+                            fn [<return_q_scale_halfneg_ $policy:lower>]() {
+                                if $cond {
+                                    let len = (<$ker>::mr() * <$ker>::nr()) as i64;
+                                    let v = (0..len).map(|i| (i - len / 2) as $tc).collect();
+                                    QScaleProblem::<$ker, $tc, $ti>::new(v, Scaler::new(-0.5f32, RoundingPolicy::$policy)).run()
+                                }
+                            }
+
+                            #[test]
+                            fn [<return_q_scale_pot_ $policy:lower>]() {
+                                if $cond {
+                                    let len = (<$ker>::mr() * <$ker>::nr()) as i64;
+                                    let v = (0..len).map(|i| (i - len / 2) as $tc).collect();
+                                    eprintln!("{:?}", &v);
+                                    QScaleProblem::<$ker, $tc, $ti>::new(v, Scaler::new(0.25f32, RoundingPolicy::$policy)).run()
+                                }
+                            }
+
+                            #[test]
+                            fn [<return_q_scale_nonpot_ $policy:lower>]() {
+                                if $cond {
+                                    let len = (<$ker>::mr() * <$ker>::nr()) as i64;
+                                    let v = (0..len).map(|i| (i - len / 2) as $tc).collect();
+                                    QScaleProblem::<$ker, $tc, $ti>::new(v, Scaler::new(1f32 / 5., RoundingPolicy::$policy)).run()
+                                }
+                            }
+
+                            #[test]
+                            fn [<return_q_scale_bigpot_ $policy:lower>]() {
+                                if $cond {
+                                    let len = (<$ker>::mr() * <$ker>::nr()) as i64;
+                                    let v = (0..len).map(|i| (i - len / 2) as $tc).collect();
+                                    QScaleProblem::<$ker, $tc, $ti>::new(v, Scaler::new(4f32, RoundingPolicy::$policy)).run()
+                                }
+                            }
+
+                            #[test]
+                            fn [<return_q_scale_bignonpot_ $policy:lower>]() {
+                                if $cond {
+                                    let len = (<$ker>::mr() * <$ker>::nr()) as i64;
+                                    let v = (0..len).map(|i| (i - len / 2) as $tc).collect();
+                                    QScaleProblem::<$ker, $tc, $ti>::new(v, Scaler::new(14., RoundingPolicy::$policy)).run()
                                 }
                             }
                         }
@@ -364,6 +357,13 @@ pub mod test {
                         }
                     }
                 }
+
+                #[test]
+                fn return_c_scale_bigpot() {
+                    if $cond {
+                        test::return_c_scale_bigpot::<$ker, $tc, $ti>()
+                    }
+                }
             }
         };
     }
@@ -688,6 +688,18 @@ pub mod test {
         fused_ops::<K, TC, TI, _>(&*v, &[FusedKerSpec::ScalarSubF(5.as_())], |_, _, c| c - five)
     }
 
+    pub fn return_c_scale_bigpot<K, TC, TI>()
+    where
+        K: MatMatMulKer<TI>,
+        TC: LADatum + AsPrimitive<TI>,
+        TI: LADatum + AsPrimitive<TC> + ScaleShiftAndRound,
+        isize: AsPrimitive<TC> + AsPrimitive<TI>,
+    {
+        let len = K::mr() * K::nr();
+        let v: Vec<TC> = (-(len as isize) / 2..).take(len).map(|f| f.as_()).collect();
+        fused_ops::<K, TC, TI, _>(&*v, &[FusedKerSpec::ShiftLeft(1)], |_, _, c| c.q_shl(1))
+    }
+
     #[derive(Debug, new)]
     pub struct QScaleProblem<K, TC, TI>
     where
@@ -697,9 +709,7 @@ pub mod test {
         i64: AsPrimitive<TC>,
     {
         pub c: Vec<TC>,
-        pub mult: i32,
-        pub shift: usize,
-        pub policy: RoundingPolicy,
+        pub scaler: Scaler,
         pub boo: std::marker::PhantomData<(K, TC, TI)>,
     }
 
@@ -717,8 +727,8 @@ pub mod test {
             let len = K::mr() * K::nr();
             (
                 proptest::collection::vec((-20i64..20).prop_map(|i| i.as_()), len..=len),
-                (1 << 29)..(1 << 30),
-                0usize..8,
+                -5i32..5,
+                prop_oneof!(Just(1f32), 0f32..1f32),
                 proptest::prop_oneof![
                     Just(Zero),
                     Just(Away),
@@ -728,11 +738,9 @@ pub mod test {
                     Just(Even)
                 ],
             )
-                .prop_map(|(c, mult, shift, policy)| QScaleProblem {
+                .prop_map(|(c, scale_pot, scale_mult, policy)| QScaleProblem {
                     c,
-                    shift,
-                    policy,
-                    mult,
+                    scaler: Scaler::new(scale_mult * 2f32.powi(scale_pot), policy),
                     boo: std::marker::PhantomData,
                 })
                 .boxed()
@@ -748,11 +756,28 @@ pub mod test {
         i64: AsPrimitive<TC>,
     {
         pub fn run(&self) {
-            fused_ops::<K, TC, TI, _>(
-                &*self.c,
-                &[FusedKerSpec::QScale(self.shift, self.policy, self.mult)],
-                |_, _, c| c.q_scale(self.mult, self.shift, self.policy),
-            )
+            if let FusedSpec::QScale(shift, policy, mult) = self.scaler.as_fused_spec() {
+                fused_ops::<K, TC, TI, _>(
+                    &*self.c,
+                    &[FusedKerSpec::QScale(shift, policy, mult)],
+                    |_, _, c| c.q_scale(self.scaler),
+                )
+            } else if let FusedSpec::RoundingShiftRight(shift, policy) = self.scaler.as_fused_spec()
+            {
+                fused_ops::<K, TC, TI, _>(
+                    &*self.c,
+                    &[FusedKerSpec::RoundingShiftRight(shift, policy)],
+                    |_, _, c| c.q_shr(shift, policy),
+                )
+            } else if let FusedSpec::ShiftLeft(shift) = self.scaler.as_fused_spec() {
+                fused_ops::<K, TC, TI, _>(
+                    &*self.c,
+                    &[FusedKerSpec::ShiftLeft(shift)],
+                    |_, _, c| c.q_shl(shift),
+                )
+            } else {
+                unreachable!()
+            }
         }
     }
 
