@@ -36,6 +36,7 @@ macro_rules! test_mmm_kernel_f32 {
                 mmm_kernel_tests!($cond, $k, f32, f32, f32, f32);
                 mmm_frame_tests!($cond, $k, f32, f32, f32, f32);
                 mmm_kernel_fuse_tests!($cond, $k, f32, f32);
+                //qmmm_kernel_fuse_tests!($cond, $k, f32, f32, f32, f32);
             }
         }
     };
@@ -51,8 +52,15 @@ macro_rules! test_mmm_kernel_i32 {
                 mmm_kernel_tests!($cond, $k, i8, i8, i8, i32);
                 mmm_kernel_fuse_tests!($cond, $k, i8, i32);
                 mmm_frame_tests!($cond, $k, i8, i8, i8, i32);
+            }
+            #[cfg(test)]
+            mod [<test_qi8_ $k>] {
                 qmmm_kernel_fuse_tests!($cond, $k, i8, i8, i8, i32);
             }
+            #[cfg(test)]
+            mod [<test_qi32_ $k>] {
+                qmmm_kernel_fuse_tests!($cond, $k, i8, i8, i32, i32);
+            }
         }
     };
 }
