@@ -136,10 +136,25 @@ where
                         }
                     }
                     FusedKerSpec::AddUnicast(tile) => add_unicast::<TI, _>(&tile, &mut ab),
+                    FusedKerSpec::ShiftLeft(shift) => {
+                        for i in 0..4 {
+                            for j in 0..4 {
+                                ab[i][j] = ab[i][j].q_shl(shift);
+                            }
+                        }
+                    }
+                    FusedKerSpec::RoundingShiftRight(shift, rp) => {
+                        for i in 0..4 {
+                            for j in 0..4 {
+                                ab[i][j] = ab[i][j].q_shr(shift, rp);
+                            }
+                        }
+                    }
                     FusedKerSpec::QScale(shift, rp, mult) => {
                         for i in 0..4 {
                             for j in 0..4 {
-                                ab[i][j] = ab[i][j].q_scale(mult, shift, rp);
+                                ab[i][j] =
+                                    ab[i][j].q_scale(Scaler::from_fuse_params(shift, rp, mult));
                             }
                         }
                     }
@@ -281,9 +296,19 @@ where
                             std::slice::from_raw_parts_mut(ab.as_ptr().offset(3) as _, 1),
                         ],
                     ),
+                    FusedKerSpec::ShiftLeft(shift) => {
+                        for i in 0..4 {
+                            ab[i][0] = ab[i][0].q_shl(shift);
+                        }
+                    }
+                    FusedKerSpec::RoundingShiftRight(shift, rp) => {
+                        for i in 0..4 {
+                            ab[i][0] = ab[i][0].q_shr(shift, rp);
+                        }
+                    }
                     FusedKerSpec::QScale(shift, rp, mult) => {
                         for i in 0..4 {
-                            ab[i][0] = ab[i][0].q_scale(mult, shift, rp);
+                            ab[i][0] = ab[i][0].q_scale(Scaler::from_fuse_params(shift, rp, mult));
                         }
                     }
                     FusedKerSpec::AddMatMul { k, pa, pb, .. } => {
@@ -417,10 +442,25 @@ where
                         }
                     }
                     FusedKerSpec::AddUnicast(tile) => add_unicast::<TI, _>(&tile, &mut ab),
+                    FusedKerSpec::ShiftLeft(shift) => {
+                        for i in 0..3 {
+                            for j in 0..2 {
+                                ab[i][j] = ab[i][j].q_shl(shift);
+                            }
+                        }
+                    }
+                    FusedKerSpec::RoundingShiftRight(shift, rp) => {
+                        for i in 0..3 {
+                            for j in 0..2 {
+                                ab[i][j] = ab[i][j].q_shr(shift, rp)
+                            }
+                        }
+                    }
                     FusedKerSpec::QScale(shift, rp, mult) => {
                         for i in 0..3 {
                             for j in 0..2 {
-                                ab[i][j] = ab[i][j].q_scale(mult, shift, rp);
+                                ab[i][j] =
+                                    ab[i][j].q_scale(Scaler::from_fuse_params(shift, rp, mult));
                             }
                         }
                     }
